// Tnc v1.00 : Programme TNC LoRa pour M5Stack.
// Ecrit par Frank LOBA (www.f4hdb.fr).
// Date création : 29/11/2022.
// Dernière mise à jour : 06/12/2022.


/**** INCLUSION DES FICHIERS EN-TETE ****/
#include <M5Stack.h>
#include <M5LoRa.h>
#include <WiFi.h>

#include <M5ModemConfig.h>
#include <M5Kiss.h>


/**** PARAMETRES SYSTEME LORA ****/
#define LORA_SCK  18
#define LORA_NSS  5
#define LORA_MISO 19  
#define LORA_MOSI 23
#define LORA_RST  26
#define LORA_IRQ  36

#define REG_MODEM_PHY_CONFIG_3_ADRESS 0x26  // See SEMTECH SX1276/77/78/79 Datasheet Rev. 6 - January 2019, page 91.
#define REG_MODEM_PHY_CONFIG_3_VALUE  0xC

#define FREQUENCE         433775000L
#define SPREADING_FACTOR  12
#define SIGNAL_BANDWIDTH  125000L
#define CODING_RATE_4     5
#define SYNC_WORD         0x12

#define LORA_START_BYTE_1 '<'
#define LORA_START_BYTE_2 0xFF
#define LORA_START_BYTE_3 0x01


/**** DEFINITION DES VARIABLES GLOBALES ****/
Modem   Modem;

String  TamponTNC2;
String  TamponAX25;
String  TamponKISS;


/**** DEFINITION DE LA FONCTION D'INITIALISATION ****/
void setup()
{
  // Initialisation du système.
  M5.begin();
  M5.Power.begin();

  // Extinction du WIFI.
  WiFi.mode(WIFI_OFF);

  // Initialisation du son.
  M5.Speaker.begin();

  // Affectation des broches.
  SPI.begin(LORA_SCK, LORA_MISO, LORA_MOSI, LORA_NSS);
  LoRa.setPins(LORA_NSS, LORA_RST, LORA_IRQ);

  // Définition de la fréquence et démarrage du LoRa
  if(!LoRa.begin(FREQUENCE))
  {
    M5.Lcd.println("Echec demarrage LoRa !");
    while(1);
  }

  // Définition des paramètres système pour LoRa.
  LoRa.setSpreadingFactor(SPREADING_FACTOR);
  LoRa.setSignalBandwidth(SIGNAL_BANDWIDTH);
  LoRa.setCodingRate4(CODING_RATE_4);
  LoRa.enableCrc();
  LoRa.setSyncWord(SYNC_WORD);

  Modem.writeRegister(REG_MODEM_PHY_CONFIG_3_ADRESS, REG_MODEM_PHY_CONFIG_3_VALUE);
}


/**** DEFINITION DE LA FONCTION D'EXECUTION ****/
void loop()
{
  M5.update();

  //**** Partie réception radio.

  // Détection d'un message reçu par radio.
  int PacketSize = LoRa.parsePacket();
 
  if (PacketSize)
  {
    // Initialisation du tampon de la trame TNC2.
    TamponTNC2 = "";
    
    // Effacement et initialisation de l'écran du M5Stack.
    M5.Lcd.clear();
    M5.Lcd.setCursor(0, 0);
    M5.Lcd.setTextSize(2);

    // Affichage de "RX" à l'écran du M5Stack
    M5.Lcd.println("**** RX ****");

    // Détection de l'en-tête LoRa.
    if(LoRa.available() && LoRa.read() == LORA_START_BYTE_1)
    if(LoRa.available() && LoRa.read() == LORA_START_BYTE_2)
    if(LoRa.available() && LoRa.read() == LORA_START_BYTE_3)
    // Lecture du paquet au format TNC2.
    while(LoRa.available())
    TamponTNC2 += (char)LoRa.read();

    // Affichage de la trame au format TNC2 à l'écran.
    M5.Lcd.println(TamponTNC2);

    // Conversion du format TNC2 au format AX25.
    TamponAX25 = TNC2toAX25(TamponTNC2);

    // La conversion a réussi.
    if(TamponAX25.length() > 0)
    {
      // Encodage de la trame AX25 avec le protocole KISS.
      TamponKISS = EncoderKISS(TamponAX25);
  
      // Ecriture dans le port série.
      Serial.print(TamponKISS);
    }
    // La conversion a échoué.
    else
    M5.Lcd.println("Echec conversion TNC2 vers AX25");
      
    // Affichage du RSSI à l'écran du M5Stack.
    M5.Lcd.print("RSSI : ");
    M5.Lcd.println(LoRa.packetRssi());
  }


  //**** Partie émission radio.

  // Détection d'un message dans le port série.
  if(Serial.available())
  {
    // Initialisation du Tampon KISS.
    TamponKISS = "";

    // Effacement et initialisation de l'écran du M5Stack.
    M5.Lcd.clear();
    M5.Lcd.setCursor(0, 0);
    M5.Lcd.setTextSize(2);

    // Affichage de "TX" à l'écran du M5Stack
    M5.Lcd.println("**** TX ****");

    // Lecture de la trame KISS dans le port série.
    while(Serial.available())
    TamponKISS += (char) Serial.read();

    // Décodage de la trame KISS.
    TamponAX25 = DecoderKISS(TamponKISS);

    // Le décodage a réussi.
    if(TamponAX25.length() > 0)
    {
      // Conversion du format AX25 au format TNC2.
      TamponTNC2 = AX25toTNC2(TamponAX25);

      // La conversion a réussi.
      if(TamponTNC2.length() > 0)
      {
        // Commencement de l'émission LoRa.
        LoRa.beginPacket();
            
        // Emission de l'en-tête LoRa.
        LoRa.write(LORA_START_BYTE_1);
        LoRa.write(LORA_START_BYTE_2);
        LoRa.write(LORA_START_BYTE_3);
            
        // Emission de la trame TNC2 en LoRa.
        LoRa.write((const uint8_t *)TamponTNC2.c_str(), TamponTNC2.length());
            
        // Fin de l'émission LoRa.
        LoRa.endPacket();

        // Affichage de la trame au format TNC2 à l'écran.
        M5.Lcd.println(TamponTNC2);     
      }
      // La conversion a échoué.
      else
      M5.Lcd.println("Echec conversion AX25 vers TNC2"); 
    }
    // Le décodage a échoué.
    else
    M5.Lcd.println("Echec décodage KISS"); 
  }

  delay(20);
}
